--
-- baleAttacher
-- Class Balle-AttachAble Tippers
--
-- @author  Geri-G
-- @date  13/07/10
--
-- Copyright (C) Geri-G

baleAttacher = {};

function baleAttacher.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Attachable, specializations);
end;

function baleAttacher:load(xmlFile)
	self.searchBales = baleAttacher.searchBales;
	self.attachBales = baleAttacher.attachBales;
	self.detachBales = baleAttacher.detachBales;
	self.OrientJoint = baleAttacher.OrientJoint;
	self.isBaleInRange = baleAttacher.isBaleInRange;
	self.place = {};
	self.place.node1 = Utils.indexToObject(self.components, getXMLString(xmlFile,"vehicle.Baletrailer#castPoint1"));		
	self.place.node2 = Utils.indexToObject(self.components, getXMLString(xmlFile,"vehicle.Baletrailer#castPoint2"));
	self.place.attacherNode = Utils.indexToObject(self.components, Utils.getNoNil(getXMLString(xmlFile,"vehicle.Baletrailer#attacherNode"),"0>"));
	self.place.highOffset = Utils.getNoNil(getXMLFloat(xmlFile,"vehicle.Baletrailer#highOffset"),4);	
	self.AttachedBales = {};
	self.attacherMod = 0;
	self.attacherModOld = 0;
	self.allowAttachment = true;
end;

function baleAttacher:update(dt)
--[[	if self:getIsActiveForInput() and self.allowAttachment then
		if InputBinding.hasEvent(InputBinding.changeAttacherMode) then
			self.attacherMod = self.attacherMod+1;
			if self.attacherMod > 3 then
				self.attacherMod = 1;
			end;
			self.attacherModOld = self.attacherMod;
		end;
		if InputBinding.hasEvent(InputBinding.attacherBales) then
			if self.attacherMod == 1 then
				self:searchBales();
			elseif self.attacherMod == 2 then
				self:detachBales();
			end;
		end;
		
	end;
	if self:getIsActive() then 
		if table.getn(self.AttachedBales) > 0 then
			self.allowFillFromAir = false;
			self.allowTipDischarge = false;
		elseif self.fillLevel > 0 then
			self.allowAttachment = false;
			self.attacherMod = 0;
		else
			self.allowFillFromAir = true;
			self.allowTipDischarge = true;
			self.allowAttachment = true;
			self.attacherMod = self.attacherModOld;
		end;
	end;]] 
end;
function baleAttacher:draw()

end;
function baleAttacher:isBaleInRange(node1,node2,Yoffset,Bale)
	local Xmax, Ymax, Zmax = getWorldTranslation(node1);
		  Xmax, Ymax, Zmax = worldToLocal(self.place.attacherNode,Xmax, Ymax, Zmax);
		  
	local Xmin, Ymin, Zmin = getWorldTranslation(node2);	
		  Xmin, Ymin, Zmin = worldToLocal(self.place.attacherNode,Xmin, Ymin, Zmin);
	
	local Xt, Yt, Zt = getWorldTranslation(Bale);
		  Xt, Yt, Zt = worldToLocal(self.place.attacherNode,Xt, Yt, Zt);
		
	if (Xt < math.max(Xmax,Xmin) and Xt > math.min(Xmax,Xmin)) and (Zt < math.max(Zmax,Zmin) and Zt > math.min(Zmax,Zmin)) and (Yt<= ((Ymax+Ymin)/2)+Yoffset and Yt>= (Ymax+Ymin)/2) then
		return true;
	else
		return false;
	end;
end;
function baleAttacher:searchBales()
	for k,v in pairs(g_currentMission.itemsToSave) do
		if v.isAttached == nil and (getUserAttribute(v.node, "isHaybale") == true or getUserAttribute(v.node, "isStrawbale") == true) then
			local isInRange = self:isBaleInRange(self.place.node1,self.place.node2,self.place.highOffset,v.node);			

			if isInRange then
				self:attachBales(v);
			end;			
		end;
	end;
end;
function baleAttacher:OrientJoint(Source, Target)	
	local xw, yw, zw = getWorldTranslation(Source);		
	local x,y,z = worldToLocal(getParent(Target), xw, yw, zw);	
	setTranslation(Target, x,y,z);			
	--JointRotation Anpassung an das Objekt
	local zX, zY, zZ = localDirectionToWorld(Source, 0,0,1);
	local zX, zY, zZ = worldDirectionToLocal(getParent(Target), zX, zY, zZ);
	local yX, yY, yZ = localDirectionToWorld(Source, 0,1,0);
	local yX, yY, yZ = worldDirectionToLocal(getParent(Target), yX, yY, yZ);
	setDirection(Target, zX, zY, zZ, yX, yY, yZ);
	return false;
end;

function baleAttacher:attachBales(bale)
	local attachedBale = {};
	attachedBale.bale = bale;
	attachedBale.AT = createTransformGroup("AT");
	link(self.place.attacherNode,attachedBale.AT);
	
	attachedBale.BaleMass = getMass(bale.node);
	setMass(bale.node,attachedBale.BaleMass*0.6);
	
	self:OrientJoint(bale.node,attachedBale.AT);

	local constr = JointConstructor:new();					
	constr:setActors(self.place.attacherNode, bale.node);
	constr:setJointTransforms(attachedBale.AT,  bale.node);
	for i=1, 3 do
		constr:setTranslationLimit(i-1, true, 0, 0);
		constr:setRotationLimit(i-1,0,0);
	end;
	attachedBale.JointIndex = constr:finalize();

	bale.isAttached = true;
	table.insert(self.AttachedBales, attachedBale);
end;

function baleAttacher:detachBales()
	for k,v in pairs(self.AttachedBales) do
		removeJoint(v.JointIndex);
		delete(v.AT);
		v.JointIndex = nil;
		setMass(v.bale.node,v.BaleMass);
		v.bale.isAttached = nil;
	end;
	self.AttachedBales = nil;
	self.AttachedBales = {};
end;



function baleAttacher:onAttach(attacherVehicle)
	--if self.attacherMod == 3 then
		self:searchBales();
		for k,v in pairs(self.attachedImplements) do
			if v.object.searchBales ~= nil then
				v.object:searchBales();
			end;
		end;
	--end;
end;
function baleAttacher:onDetach()
	--if self.attacherMod == 3 then
		self:detachBales();
		for k,v in pairs(self.attachedImplements) do
			if v.object.detachBales ~= nil then
				v.object:detachBales();
			end;
		end;
	--end;
end;
function baleAttacher:onActivate()
	--if self.attacherMod == 3 then
		self:searchBales();
		for k,v in pairs(self.attachedImplements) do
			if v.object.searchBales ~= nil then
				v.object:searchBales();
			end;
		end;
	--end;
end;
function baleAttacher:onDeactivate()
	--if self.attacherMod == 3 then
		self:detachBales();
		for k,v in pairs(self.attachedImplements) do
			if v.object.detachBales ~= nil then
				v.object:detachBales();
			end;
		end;
	--end;
end;
function baleAttacher:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not resetVehicles then

	end;
    return BaseMission.VEHICLE_LOAD_OK;
end;

function baleAttacher:getSaveAttributesAndNodes(nodeIdent)
   -- local attributes = ' side="'..tostring(self.currentSide)..'"';
    local attributes = nil;
    local node = nil;
    return attributes, node;
end;
function baleAttacher:delete()

end;

function baleAttacher:mouseEvent(posX, posY, isDown, isUp, button)
end;

function baleAttacher:keyEvent(unicode, sym, modifier, isDown)
end;
